import click
import operator
from icheckin import constants
from os.path import exists

def exist():
   '''
   -> boolean: whether the file exists
   '''
   return exists(constants.PATH)

def maximum():
   '''
   -> int: maximum credentials allowed
   '''
   return constants.MAX

def primary():
   '''
   -> (student id, password)  : Primary credentials 
   -> ()                      : No primary credentials
   '''
   credsList = read()
   if len(credsList) != 0:
      return credsList[0]
   else:
      return ()

def empty():
   '''
   -> True  : Empty credentials
   -> False : Non-empty credentials
   '''
   return len(read()) == 0

def display(credsList, title='Credentials', noPrimary=False, numbering=False):
   click.echo()
   click.echo(title)
   click.echo('-'*len(title))
   empty = True if len(credsList) == 0 else False
   if not empty:
      studentIds = map(operator.itemgetter(0), credsList)
      spacing = max(list(map(lambda x: len(x), studentIds))) + 1
      digits = len(str(len(credsList)-1))
      for i in range(len(credsList)):
         if numbering:
            click.echo('[%s] ' % str(i).rjust(digits), nl=False)
         click.echo('%s: %s' % (
            credsList[i][0].ljust(spacing),
            credsList[i][1]), nl=False)
         if i == 0:
            click.echo('%s' % '' if noPrimary else ' (Primary)')
         else:
            click.echo()
   else:
      click.echo('(Empty)')
   click.echo()

def prompt(prompt1='Student id', prompt2='Password'):
   ''' 
   -> tuple: (student id, password)
   '''
   studentId = input(prompt1+': ')
   password = input(prompt2+': ')
   return (studentId, password)

def read():
   '''
   -> list: [(student id, password), (), ...]
   '''
   credsList = []
   if exist():
      with open(constants.PATH, 'r') as file:
         lines = list(map(str.strip, file.readlines()))
         if len(lines) % 2 == 0:
            for i in range(0, len(lines), 2):
               credsList.append((lines[i], lines[i+1]))
   return credsList

def write(credsList, _mode='w'):
   assert len(credsList) <= maximum()
   with open(constants.PATH, _mode) as file:
      for creds in credsList:
         file.write(creds[0]+'\n')
         file.write(creds[1]+'\n')

def append(credsList):
   assert exist()
   assert len(read()) + len(credsList) <= maximum()
   write(credsList, _mode='a')

def delete(indexList):
   assert exist()
   creds = read()
   for index in sorted(indexList, reverse=True):
      assert index >= 0 and index < len(creds)
      creds.pop(index)
   write(creds)

def clear():
   assert exist()
   write([])


import click
from icheckin import credentials

@click.command('show', short_help='show saved credentials')
def command():
   if credentials.exist():
      credentials.display(credentials.read())
   else:
      click.echo('No saved credentials.')


from re import match
from os import listdir
from os.path import realpath, dirname

path = dirname(realpath(__file__))
files = list(filter(lambda x: match('[a-z]+\.py', x), listdir(path)))
# For from icheckin.commands import *
__all__ = list(map(lambda x: x.strip('.py'), files))
# For entry point to access
MODULES = __all__


import sys
import click
from icheckin import credentials

@click.command('add', short_help='add to existing credentials')
def command():
   # Check primary credentials
   if credentials.primary() != ():
      # Display credentials
      creds = credentials.read()
      credentials.display(creds, 'Existing Credentials')
      # Prompt to add
      count = len(creds)
      creds = []
      if count != credentials.maximum():
         click.echo('[Enter twice to stop adding]')  
         while True:
            cred = credentials.prompt()
            if cred == ('', ''):
               break
            else:
               if '' not in cred:
                  creds.append(cred)
                  count += 1
                  # Check max
                  if count != credentials.maximum():
                     click.echo('[Next]')
                  else:
                     click.echo('[Maximum credentials reached: %s]' % 
                        str(credentials.maximum()))
                     break
               else:
                  click.echo('Try again.')
      else:
         click.echo('Maximum credentials already.')
         sys.exit(0)
      # Confirm
      if len(creds) != 0:
         credentials.display(creds, 'Credentials to Add', noPrimary=True)
         if click.confirm('Confirm?'):
            credentials.append(creds)
            click.echo('Successfully added.')
         else:
            click.echo('Cancelled.')
   else:
      click.echo('Save primary credentials first before adding.')


import sys
import click
from halo import Halo
from icheckin import credentials, connections

@click.command('code', short_help='check-in with code')
@click.argument('checkincode')
@click.option('--multi', is_flag=True, help='Check-in multiple credentials.')
def command(checkincode, multi):
   # Check primary credentials
   primary = credentials.primary()
   if primary != ():
      click.echo('\nChecking in for:')
      for cred in credentials.read():
         spinner = Halo(text=cred[0])
         spinner.start()
         status = connections.checkin(cred, checkincode)
         if status == 0:
            spinner.succeed('%s - Successful' % cred[0])
         elif status == 1:
            spinner.fail('%s - No internet connection' % cred[0])
            sys.exit(0)
         elif status == 2:
            spinner.fail('%s - Invalid credentials' % cred[0])
         elif status == 3:
            spinner.fail('%s - Not connected to SunwayEdu Wi-Fi' % cred[0])
            sys.exit(0)
         elif status == 4:
            spinner.fail('%s - Invalid code' % cred[0])
            sys.exit(0)
         elif status == 5:
            spinner.fail('%s - Wrong class' % cred[0])
         elif status == 6:
            spinner.fail('%s - Already checked-in' % cred[0])
         if not multi:
            break
      click.echo('\nDone.')
   else:
      click.echo('Save primary credentials first before checking-in.')


import requests
from icheckin import constants

def update_status():
   ''' 
   -< True  : Version is test; no update needed
   -> False : Check for update failed
   -> str   : Latest version to update to
   '''
   try:
      r = requests.get(constants.UPDATES, timeout=5)
   except (requests.ConnectionError, requests.ConnectTimeout, 
      requests.ReadTimeout):
      return False
   else:
      latestVersion = r.json()['latest_version']
      if constants.VERSION == latestVersion:
         return True
      else:
         return latestVersion

def checkin(cred, code):
   '''
   -> 0: Successful check-in
   -> 1: No internet connection
   -> 2: Invalid credentials
   -> 3: Not connected to SunwayEdu Wi-Fi
   -> 4: Invalid code
   -> 5: Wrong class
   -> 6: Already checked-in
   '''
   # Start a session
   session = requests.Session()
   # Login to iZone
   payload = {
      'form_action': 'submitted',
      'student_uid': cred[0],
      'password': cred[1],
   }
   try:
      r = session.post(constants.LOGIN, data=payload)
   except requests.ConnectionError:
      return 1
   if not r.history:
      return 2
   # Check for SunwayEdu Wi-Fi
   try:
      r = requests.get(constants.WIFI, timeout=2)
   except requests.ConnectTimeout:
      return 3
   except requests.ConnectionError:
      return 1
   # Check-in with code
   try:
      r = session.post(constants.CHECKIN, data={'checkin_code': code}, 
         timeout=2)
   except (requests.ReadTimeout, requests.ConnectionError):
      return 1
   if 'Checkin code not valid.' in r.text or \
      'The specified URL cannot be found.' in r.text:
      return 4
   if 'You cannot check in to a class you are not a part of.' in r.text:
      return 5
   if 'You have already checked in' in r.text:
      return 6
   return 0


import sys
import click
from icheckin import credentials

@click.command('remove', short_help='remove credentials')
def command():
   if credentials.exist():
      creds = credentials.read()
      credentials.display(creds, 'Existing Credentials', 
         numbering=True)
      # Prompt for indexes        
      if not credentials.empty():
         click.echo('[Comma for multiple values]')
         while True:
            response = input('Indexes: ')
            if response == '':
               sys.exit()
            else:
               indexes = list(map(lambda x: x.strip(), response.split(',')))
               try:
                  indexes = list(map(lambda x: int(x), indexes))
               except ValueError:
                  click.echo('Try again.')
                  continue
               for index in indexes:
                  if index < 0 or index >= len(creds):
                     click.echo('Try again.')
                     break
               else:
                  break
         # Remove credentials
         credsToRemove = []
         indexes = sorted(list(set(indexes)))
         click.echo(indexes)
         for index in indexes:
            credsToRemove.append(creds[index])
         credentials.display(credsToRemove, 'Credentials to Remove', noPrimary=True)
         if click.confirm('Confirm?'):            
            credentials.delete(indexes)
            click.echo('Successfully removed.')
      else:
         click.echo('Nothing to remove.')
   else:
      click.echo('No saved credentials.')



import click
from icheckin import credentials

@click.command('clear', short_help='clear all credentials')
def command():
   if credentials.exist():
      credentials.display(credentials.read(), 'Credentials to Clear')  
      if not credentials.empty():
         if click.confirm('Confirm?'):
            credentials.clear()
            click.echo('Successfully cleared.')
      else:
         click.echo('Nothing to clear.')
   else:
      click.echo('No saved credentials.')



from setuptools import setup, find_packages

setup(
	name = 'icheckin',
	packages = find_packages(),
	install_requires = [
		'requests==2.18.1',
		'click==6.7',
		'halo==0.0.7'
	],
	version = '1.3.5',
	description = 'A console application for Sunway University\'s iCheckin',
	author = 'Marcus Mu',
	author_email = 'chunkhang@gmail.com',
	license = 'UNLICENSE',
	url = 'https://github.com/chunkhang/icheckin',
	keywords = [
		'icheckin', 
		'sunway'
	], 
	classifiers = [
		'Intended Audience :: End Users/Desktop',
		'Programming Language :: Python :: 3 :: Only',
		'Environment :: Console'
	],
	entry_points = {
		'console_scripts': [
			'icheckin=icheckin.icheckin:cli'
		]
	}
)


import click
from icheckin.commands import *
from icheckin.commands import MODULES

@click.group()
def cli():
   pass

# Add all commands to cli
for module in MODULES:
	cli.add_command(getattr(locals()[module], 'command'))
   
if __name__ == '__main__':
	cli()


import sys
import click
from icheckin import credentials

@click.command('save', short_help='save new credentials')
def command():
   # Display existing credentials
   question = 'Save new credentials?'
   message = 'Successfully saved.'
   if credentials.exist():
      question = 'Overwrite credentials?'
      message = 'Successfully overwritten.'
      creds = credentials.read()
      credentials.display(creds, 'Existing Credentials')
         
   # Save credentials
   if click.confirm(question):
      creds = []
      # Primary credentials
      click.echo('\n[Enter twice to exit]')
      while True:
         cred = credentials.prompt('Primary student id',
            'Primary password')
         if cred == ('', ''):
            sys.exit(0)
         else:
            if '' not in cred:
               creds.append(cred)
               break
            else:
               click.echo('Try again.')      
      # Secondary credentials
      if click.confirm('\nAdd more credentials?'):
         click.echo('\n[Enter twice to stop adding]')
         while True:
            cred = credentials.prompt()
            if cred == ('', ''):
               break
            else:
               if '' not in cred:
                  creds.append(cred)
                  # Check max
                  if len(creds) != credentials.maximum():
                     click.echo('[Next]')
                  else:
                     click.echo('[Maximum credentials reached: %s]' % 
                        str(credentials.maximum()))
                     break
               else:
                  click.echo('Try again.')
      # Confirm
      credentials.display(creds, 'New Credentials')
      if click.confirm('Confirm?'):
         credentials.write(creds)
         click.echo(message)
      else:
         click.echo('Cancelled.')




import click
from halo import Halo
from icheckin import connections, constants

@click.command('update', short_help='check for updates')
def command():
   click.echo('Current version: %s' % constants.VERSION)
   spinner = Halo(text='Checking for updates')
   spinner.start()
   update = connections.update_status()
   if update == True:
      spinner.succeed('Up to date.')
   elif update == False:
      spinner.fail('Failed.')
   else:
      spinner.warn('A newer version (%s) is available.' % update)
      click.echo('$ pip install --upgrade icheckin')



from os.path import join, expanduser

# Version
VERSION = '1.3.5'
# URLs
UPDATES = 'http://chunkhang.pythonanywhere.com/icheckin/api'
LOGIN = 'https://izone.sunway.edu.my/login'
WIFI = 'https://icheckin.sunway.edu.my/otp/CheckIn/isAlive/' +\
	'CuNv9UV2rXg4WtAsXUPNptg6gWQTZ52w'
CHECKIN = 'https://izone.sunway.edu.my/icheckin/iCheckinNowWithCode'
# Credentials
PATH = join(expanduser("~"), '.icheckin-credentials')
MAX = 20